@@ -1,18 +1,17 @@
 parse_IETF_language_tag <-
-function(x, expand = FALSE)
+function(x, expand = FALSE, strict = TRUE)
 {
     n <- length(x)
     y <- rep.int(list(character()), n)
     names(y) <- x
+    
+    pos <- seq_along(x)    
 
     ## How nice should we be?
     ## Allow for empty or missing elements ...
-
-    pos <- seq_along(x)
-    if(any(ind <- (is.na(x) | (x == "")))) {
-        pos <- pos[!ind]
-        x <- x[pos]
-    }
+    ind <- which(!is.na(x) & nzchar(x))
+    pos <- pos[ind]
+    x <- x[ind]
 
     ## See <https://www.ietf.org/rfc/rfc4646.txt>.
     ## Language tags can be of the form (in ABNF, see
@@ -30,8 +29,9 @@ function(x, expand = FALSE)
                         IANA_language_subtag_registry_grandfathered_table$Tag))
     if(any(ind)) {
         y[pos[ind]] <- as.list(sprintf("Grandfathered=%s", x[ind]))
-        x[ind] <- ""
-        pos <- pos[!ind]
+        ind <- which(!ind)
+        pos <- pos[ind]
+        x <- x[ind]
     }
 
     if(length(pos)) {    
@@ -41,8 +41,9 @@ function(x, expand = FALSE)
             y[pos[ind]] <-
                 as.list(sprintf("Privateuse=%s",
                                 substring(x[ind], 3L)))
-            x[ind] <- ""
-            pos <- pos[!ind]
+            ind <- which(!ind)
+            pos <- pos[ind]
+            x <- x[ind]
         }
     }
 
@@ -85,19 +86,18 @@ function(x, expand = FALSE)
     if(length(pos)) {
         pat <- sprintf("^(%s)(-.*|$)", re_language)
         ind <- grepl(pat, x, perl = TRUE)
-        if(!all(ind)) {
-            bad <- which(!ind)
-            x[bad] <- ""
-        }
-        y[pos[ind]] <-
-            lapply(strsplit(sub(pat, "\\1", x[ind], perl = TRUE),
+        bad <- pos[!ind]
+        pos <- pos[ind]
+        x <- x[ind]
+        y[pos] <-
+            lapply(strsplit(sub(pat, "\\1", x, perl = TRUE),
                             "-", fixed = TRUE),
                    function(e) {
                        c(sprintf("Language=%s", e[1L]),
                          sprintf("Extension=%s", e[-1L]))
                    })
-        x[ind] <- sub(pat, "\\3", x[ind], perl = TRUE)
-        ind <- nzchar(x)
+        x <- sub(pat, "\\3", x, perl = TRUE)
+        ind <- which(nzchar(x))
         pos <- pos[ind]
         x <- x[ind]
     }
@@ -115,7 +115,7 @@ function(x, expand = FALSE)
                         sprintf("Script=%s",
                                 sub(pat, "\\1", x[ind], perl = TRUE)))
                 x[ind] <- sub(pat, "\\2", x[ind], perl = TRUE)
-                ind <- nzchar(x)
+                ind <- which(nzchar(x))
                 pos <- pos[ind]
                 x <- x[ind]
                 if(!length(x)) break
@@ -130,7 +130,7 @@ function(x, expand = FALSE)
                         sprintf("Region=%s",
                                 sub(pat, "\\1", x[ind], perl = TRUE)))
                 x[ind] <- sub(pat, "\\2", x[ind], perl = TRUE)
-                ind <- nzchar(x)
+                ind <- which(nzchar(x))
                 pos <- pos[ind]
                 x <- x[ind]
                 if(!length(x)) break
@@ -145,7 +145,7 @@ function(x, expand = FALSE)
                         sprintf("Variant=%s",
                                 sub(pat, "\\1", x[ind], perl = TRUE)))
                 x[ind] <- sub(pat, "\\2", x[ind], perl = TRUE)
-                ind <- nzchar(x)
+                ind <- which(nzchar(x))
                 pos <- pos[ind]
                 x <- x[ind]
             }
@@ -166,7 +166,7 @@ function(x, expand = FALSE)
                                 sub(pat, "\\1\\2", x[ind], perl = TRUE)))
                 ## </NOTE>
                 x[ind] <- sub(pat, "\\3", x[ind], perl = TRUE)
-                ind <- nzchar(x)
+                ind <- which(nzchar(x))
                 pos <- pos[ind]
                 x <- x[ind]
             }
@@ -190,38 +190,40 @@ function(x, expand = FALSE)
     }
 
     ## Be a nuisance: singletons for extensions must not be duplicated.
-    ind <- as.logical(lapply(y, function(e) {
-        e <- grep("^Extension=", e, value = TRUE)
-        if(!length(e)) return(FALSE)
-        any(duplicated(sub("^Extension=(.).*", "\\1", e)))
-    }))
+    ind <- vapply(y,
+                  function(e) {
+                      e <- grep("^Extension=", e, value = TRUE)
+                      if(!length(e)) return(FALSE)
+                      any(duplicated(sub("^Extension=(.).*", "\\1", e)))
+                  },
+                  NA)
     if(any(ind))
         bad <- c(bad, which(ind))
     if(any(ind <- nzchar(x))) {
         bad <- c(bad, pos[ind])
     }
-    if(length(bad)) {
+    
+    if(length(bad) && strict) {
         stop("Invalid language tag(s):",
-             paste("\n ", names(y)[bad], collapse = " "),
+             paste("\n ", sQuote(names(y)[bad]), collapse = " "),
              call. = FALSE)
     }
 
     if(!expand) return(y)
 
-    x <- tolower(unlist(y))
-    pos <- match(x, IANA_language_subtag_registry$Index)
-    z <- IANA_language_subtag_registry$Description[pos]
+    n <- lengths(y)
+    i <- (n > 0L)
+    x <- tolower(unlist(y, use.names = FALSE))
+    p <- match(x, IANA_language_subtag_registry$Index)
+    z <- IANA_language_subtag_registry$Description[p]
     ## Special case private use ranges.
     if(!all(lengths(z))) {
-        pos <- match(x,
-                     IANA_language_subtag_registry_private_use_index_table)
-        z[pos > 0L | grepl("^privateuse=", x)] <- "Private use"
+        p <- match(x, IANA_language_subtag_registry_private_use_index_table)
+        z[p > 0L | grepl("^privateuse=", x)] <- "Private use"
     }
-    z <- Map(`names<-`,
-             split(z, rep.int(seq_along(y), lengths(y))),
-             y)
-    names(z) <- names(y)
-    z
+    z <- Map(`names<-`, split(z, rep.int(seq_along(y), n)), y[i])
+    y[i] <- z
+    y
 }
 
 get_IANA_language_subtag_registry <-
